%nuclear_binding_energy_surface
% Function which imports nuclear binding energy data from an Excel Sheet
% and plots this over an energy vs proton vs neatoron number surface
% described by the Liquid Drop model.
%
% LAST UPDATED by Andy French October 2012
%
% References:
%
% [1]  Woan, G., "The Cambridge Handbook of Physics Formulas."
%      Cambridge University Press 2000. pp103.
%
% [2]  "The Ame2003 atomic mass evaluation (II)"  by G.Audi, A.H.Wapstra and C.Thibault
%      Nuclear Physics A729 p. 337-676, December 22, 2003.
%
% Note: 1 atomic mass unit (u) = 1.66053886  10^-27 kilograms
%       1MeV = 1.60217646  10^-13 joules

function nuclear_binding_energy_surface

nuclear_binding_energy_surfaces('2D')
nuclear_binding_energy_surfaces('3D')

%%

function nuclear_binding_energy_surfaces(view_type)

%Set surface limits
B_limits = [-5,10];
NZ_frac_diff_limits = [-40,150];

%

%Import Excel sheet of binding energy data
[num,txt,D] = xlsread('nuclear binding energy datasheet.xls');

%Get dimensions of cell array D
dim = size(D);

%Initialise Neutron number (N) and Proton number (Z) arrays.
%Also intitialise Binding energy (B) /MeV, Atomic mass (M), /u and mass
%excess dM /u
N = zeros(dim(1)-2,1);
Z = zeros(dim(1)-2,1);
B = zeros(dim(1)-2,1);
M = zeros(dim(1)-2,1);
dM = zeros(dim(1)-2,1);

%Extract data from cell array D. Ignore first two rows which are headers
for n=1:dim(1)-2
    N(n) = D{n+2,3};
    Z(n) = D{n+2,4};
    B(n) = D{n+2,10}/1000;
    M(n) = D{n+2,16}*1e-6;
    dM(n) = D{n+2,8}*1e-6;
end

%Determine extent of N and Z numbers
Nmax = max(N);
Zmax = max(Z);

%Generate array of Binding energies /MeV using the Liquid Drop model
NN = 0:Nmax;
ZZ = 0:Zmax;
[ZZ,NN] = meshgrid(ZZ,NN);
BB = liquid_drop_model( NN,ZZ );

%Interpolate Binding energy data points
B_interp = TriScatteredInterp(Z,N,B);

%Compute curvature of Binding energy surface
dZ = 1;
dN = 1;
K = del2( B_interp(ZZ,NN),1 );
KK = del2( BB,dZ,dN );

%

% Binding energy surface %

%Plot Binding energy surface from Liquid Drop Model
figure('name','Nuclear Binding Energy','color',[1 1 1])
surf( ZZ,100*(NN-ZZ)./ZZ,BB,BB,'facealpha',0.4 );
shading interp
if strcmp(view_type,'2D'); grid off; box on; axis tight; end
colorbar('location','eastoutside','fontsize',8);
title('Nuclear Binding Energy per nucleon /MeV')
xlabel('Z')
ylabel('100*(N-Z)/Z')
zlabel('B /MeV')
interp_colormap(1000);
ylim(NZ_frac_diff_limits)
zlim(B_limits)
if strcmp( view_type,'2D' )
    view(2);
end
caxis([mean(B)-std(B),mean(B)+std(B)])

%Plot Interpolated Binding Energy surface
hold on;
surf( ZZ,100*(NN-ZZ)./ZZ,B_interp(ZZ,NN),B_interp(ZZ,NN),'facealpha',0.7);
shading interp

%Add actual values
plot3( Z, 100*(N-Z)./Z, B,'k.','markersize',3);

%Save a copy of the surface
print(gcf,['Nuclear Binding Energy ',view_type],'-dpng','-r300');
close(gcf)

%

% Binding Energy surface with curvature colouring %

%Plot Binding energy surface from Liquid Drop Model
figure('name','Nuclear Binding Energy','color',[1 1 1])
surf( ZZ,100*(NN-ZZ)./ZZ,BB,KK,'facealpha',0.4 );
shading interp
if strcmp(view_type,'2D'); grid off; box on; axis tight; end
colorbar('location','eastoutside','fontsize',8);
title({'Nuclear Binding Energy per nucleon /MeV','Colour is Binding Energy surface curvature'})
xlabel('Z')
ylabel('100*(N-Z)/Z')
zlabel('B /MeV')
cmap = [
    1,0.2,0.2;
    0.2,0.2,1;
    0.2,1,0.2];
colormap(cmap);
interp_colormap(1000);
ylim(NZ_frac_diff_limits)
zlim(B_limits)
if strcmp( view_type,'2D' )
    view(2);
end
k = reshape(K,[1,numel(K)]);
k(isnan(k)) = [];
caxis([mean(k)-std(k),mean(k)+std(k)])

%Plot Interpolated Binding Energy surface
hold on;
surf( ZZ,100*(NN-ZZ)./ZZ,B_interp(ZZ,NN),K,'facealpha',0.7);
shading interp

%Add actual values
plot3( Z, 100*(N-Z)./Z, B,'k.','markersize',3);

%Save a copy of the surface
print(gcf,['Nuclear Binding Energy curvature ',view_type],'-dpng','-r300');
close(gcf)


%%

%Binding energy /MeV per nucleon as a function of Neutron number (N) and Proton number (Z)
function B = liquid_drop_model( N,Z )

%Constants /MeV
a_v = 15.8;
a_s = 18.0;
a_c = 0.72;
a_a = 23.5;
a_p = 33.5;

%Mass number
A = Z + N;

%Pairing term
dA = zeros(size(A));
both_even = find(( rem(Z,2)==0 ) & ( rem(N,2)==0 ) );
both_odd = find(( rem(Z,2)~=0 ) & ( rem(N,2)~=0 ) );
dA(both_even) = a_p*A(both_even).^(-3/4);
dA(both_odd) = -a_p*A(both_odd).^(-3/4);

%Compute Binding Energy per nucleon /MeV
B = a_v*A - a_s*A.^(2/3) -a_c*(Z.^2)./(A.^(1/3)) - a_a*((N-Z).^2)./A + dA;
B = B./A;

%End of code